@@ -23,7 +23,7 @@ from .utils import log, make_str
 def save_gdf_shapefile(gdf, filename=None, folder=None):
     """
     Save GeoDataFrame as an ESRI shapefile.
-    
+
     Parameters
     ----------
     gdf : GeoDataFrame
@@ -32,33 +32,33 @@ def save_gdf_shapefile(gdf, filename=None, folder=None):
         what to call the shapefile (file extensions are added automatically)
     folder : string
         where to save the shapefile, if none, then default folder
-    
+
     Returns
     -------
     None
     """
     if folder is None:
         folder = globals.data_folder
-    
+
     if filename is None:
         filename = make_shp_filename(gdf.gdf_name)
-    
+
     # give the save folder a filename subfolder to make the full path to the files
     folder_path = '{}/{}'.format(folder, filename)
-    
+
     # if the save folder does not already exist, create it with a filename subfolder
     if not os.path.exists(folder_path):
         os.makedirs(folder_path)
     gdf.to_file(folder_path)
     if not hasattr(gdf, 'gdf_name'):
         gdf.gdf_name = 'unnamed'
-    log('Saved the GeoDataFrame "{}" as shapefile "{}"'.format(gdf.gdf_name, folder_path)) 
-    
-    
+    log('Saved the GeoDataFrame "{}" as shapefile "{}"'.format(gdf.gdf_name, folder_path))
+
+
 def save_graph_shapefile(G, filename='graph', folder=None, encoding='utf-8'):
     """
     Save graph nodes and edges as ESRI shapefiles to disk.
-    
+
     Parameters
     ----------
     G : networkx multidigraph
@@ -68,7 +68,7 @@ def save_graph_shapefile(G, filename='graph', folder=None, encoding='utf-8'):
         the folder to contain the shapefiles, if None, use default data folder
     encoding : string
         the character encoding for the saved shapefiles
-    
+
     Returns
     -------
     None
@@ -77,17 +77,15 @@ def save_graph_shapefile(G, filename='graph', folder=None, encoding='utf-8'):
     start_time = time.time()
     if folder is None:
         folder = globals.data_folder
-    
-    # convert directed graph G to an undirected graph for saving as a shapefile
-    G_save = G.copy()
-    G_save = get_undirected(G_save)
 
-    
+    # convert directed graph G to an undirected graph for saving as a shapefile
+    G_save = get_undirected(G.copy())
+
     # create a GeoDataFrame of the nodes and set CRS
     nodes = {node:data for node, data in G_save.nodes(data=True)}
     gdf_nodes = gpd.GeoDataFrame(nodes).T
     gdf_nodes.crs = G_save.graph['crs']
-    
+
     # create a geometry column then drop the x and y columns
     gdf_nodes['geometry'] = gdf_nodes.apply(lambda row: Point(row['x'], row['y']), axis=1)
     gdf_nodes = gdf_nodes.drop(['x', 'y'], axis=1)
@@ -96,7 +94,7 @@ def save_graph_shapefile(G, filename='graph', folder=None, encoding='utf-8'):
     gdf_nodes['osmid'] = gdf_nodes['osmid'].astype(np.int64)
     for col in [c for c in gdf_nodes.columns if not c == 'geometry']:
         gdf_nodes[col] = gdf_nodes[col].fillna('').map(make_str)
-    
+
     # create a list to hold our edges, then loop through each edge in the graph
     edges = []
     for u, v, key, data in G_save.edges(keys=True, data=True):
@@ -111,7 +109,7 @@ def save_graph_shapefile(G, filename='graph', folder=None, encoding='utf-8'):
             point_u = Point((G_save.node[u]['x'], G_save.node[u]['y']))
             point_v = Point((G_save.node[v]['x'], G_save.node[v]['y']))
             edge_details['geometry'] = LineString([point_u, point_v])
-        
+
         edges.append(edge_details)
 
     # create a geodataframe from the list of edges and set the CRS
@@ -121,22 +119,22 @@ def save_graph_shapefile(G, filename='graph', folder=None, encoding='utf-8'):
     # make everything but geometry column a string
     for col in [c for c in gdf_edges.columns if not c == 'geometry']:
         gdf_edges[col] = gdf_edges[col].fillna('').map(make_str)
-        
+
     # if the save folder does not already exist, create it with a filename subfolder
     folder = '{}/{}'.format(folder, filename)
     if not os.path.exists(folder):
         os.makedirs(folder)
-    
+
     # save the nodes and edges as separate ESRI shapefiles
     gdf_nodes.to_file('{}/nodes'.format(folder), encoding=encoding)
     gdf_edges.to_file('{}/edges'.format(folder), encoding=encoding)
     log('Saved graph "{}" to disk as shapefiles at "{}" in {:,.2f} seconds'.format(G_save.name, folder, time.time()-start_time))
-    
-    
+
+
 def save_graphml(G, filename='graph.graphml', folder=None):
     """
     Save graph as GraphML file to disk.
-    
+
     Parameters
     ----------
     G : networkx multidigraph
@@ -144,16 +142,16 @@ def save_graphml(G, filename='graph.graphml', folder=None):
         the name of the graphml file (including file extension)
     folder : string
         the folder to contain the file, if None, use default data folder
-    
+
     Returns
     -------
     None
     """
-    
+
     start_time = time.time()
     if folder is None:
         folder = globals.data_folder
-    
+
     # create a copy and convert all the node/edge attribute values to string or it won't save
     G_save = G.copy()
     for dict_key in G_save.graph:
@@ -167,50 +165,50 @@ def save_graphml(G, filename='graph.graphml', folder=None):
         for dict_key in data:
             # convert all the edge attribute values to strings
             data[dict_key] = make_str(data[dict_key])
-                
+
     if not os.path.exists(folder):
         os.makedirs(folder)
-    
+
     nx.write_graphml(G_save, '{}/{}'.format(folder, filename))
     log('Saved graph "{}" to disk as GraphML at "{}/{}" in {:,.2f} seconds'.format(G_save.name, folder, filename, time.time()-start_time))
-    
-    
+
+
 def load_graphml(filename, folder=None):
     """
     Load a GraphML file from disk and convert the node/edge attributes to correct data types.
-    
+
     Parameters
     ----------
     filename : string
         the name of the graphml file (including file extension)
     folder : string
         the folder containing the file, if None, use default data folder
-    
+
     Returns
     -------
     networkx multidigraph
     """
     start_time = time.time()
-    
+
     # read the graph from disk
     if folder is None:
         folder = globals.data_folder
     path = '{}/{}'.format(folder, filename)
     G = nx.MultiDiGraph(nx.read_graphml(path, node_type=int))
-    
+
     # convert graph crs attribute from saved string to correct dict data type
     G.graph['crs'] = ast.literal_eval(G.graph['crs'])
-     
+
     if 'streets_per_node' in G.graph:
         G.graph['streets_per_node'] = ast.literal_eval(G.graph['streets_per_node'])
-        
+
     # convert numeric node tags from string to numeric data types
     log('Converting node and edge attribute data types')
     for node, data in G.nodes(data=True):
         data['osmid'] = int(data['osmid'])
         data['x'] = float(data['x'])
         data['y'] = float(data['y'])
-    
+
     # convert numeric, bool, and list node tags from string to correct data types
     for u, v, key, data in G.edges(keys=True, data=True):
 
@@ -235,73 +233,142 @@ def load_graphml(filename, folder=None):
         # if geometry attribute exists, load the string as well-known text to shapely LineString
         if 'geometry' in data:
             data['geometry'] = wkt.loads(data['geometry'])
-    
+
     # remove node_default and edge_default metadata keys if they exist
     if 'node_default' in G.graph:
         del G.graph['node_default']
     if 'edge_default' in G.graph:
         del G.graph['edge_default']
-    
+
     log('Loaded graph with {:,} nodes and {:,} edges in {:,.2f} seconds from "{}"'.format(len(list(G.nodes())),
                                                                                           len(list(G.edges())),
                                                                                           time.time()-start_time,
                                                                                           path))
     return G
-    
 
+
+def is_duplicate_edge(data, data_other):
+    """
+    Check if two edge data dictionaries are the same based on OSM ID and geometry.
+
+    Parameters
+    ----------
+    data : dict
+        the first edge's data
+    data_other : dict
+        the second edge's data
+
+    Returns
+    -------
+    is_dupe : bool
+    """
+    
+    is_dupe = False
+    
+    if data['osmid'] == data_other['osmid']:
+        # if they have the same OSM IDs
+        if ('geometry' in data) and ('geometry' in data_other):
+            # if both edges have a geometry attribute
+            if is_same_geometry(data, data_other):
+                # if their edge geometries have the same coordinates
+                is_dupe = True
+        elif ('geometry' in data) and ('geometry' in data_other):
+            # if neither edge has a geometry attribute
+            is_dupe = True
+        else:
+            # if one edge has geometry attribute but the other doesn't, keep it
+            pass
+
+    return is_dupe
+    
+    
+def is_same_geometry(data, data_other):
+    """
+    Check if LineString geometries in two edge data dicts are the same, in normal or reversed order of points.
+
+    Parameters
+    ----------
+    data : dict
+        the first edge's data
+    data_other : dict
+        the second edge's data
+
+    Returns
+    -------
+    bool
+    """
+    
+    # extract geometries from each edge data dict
+    geom1 = [list(coords) for coords in data['geometry'].xy]
+    geom2 = [list(coords) for coords in data_other['geometry'].xy]
+    
+    # reverse the first edge's list of x's and y's to look for a match in either order
+    geom1_r = [list(reversed(list(coords))) for coords in data['geometry'].xy]
+    
+    # if the edge's geometry matches its reverse's geometry in either order, return True
+    return (geom1 == geom2 or geom1_r == geom2)
+    
+    
 def get_undirected(G):
     """
-    Convert a directed graph to an undirected graph that maintains parallel edges in opposite directions if geometries differ.
-    
+    Convert a directed graph to an undirected graph that maintains parallel edges if geometries differ.
+
     Parameters
     ----------
     G : networkx multidigraph
-    
+
     Returns
     -------
     networkx multigraph
     """
-    # set from/to nodes and then make undirected
+    
+    start_time = time.time()
+    
+    # set from/to nodes before making graph undirected
     G = G.copy()
     for u, v, key in G.edges(keys=True):
         G.edge[u][v][key]['from'] = u
         G.edge[u][v][key]['to'] = v
     
-    G_undir = G.to_undirected(reciprocal=False)
+    # now convert multidigraph to a multigraph, retaining all edges for now
+    H = nx.MultiGraph()
+    H.name = G.name
+    H.add_nodes_from(G)
+    H.add_edges_from(G.edges(keys=False, data=True))
+    H.graph = G.graph
+    H.node = G.node
     
-    # if edges in both directions (u,v) and (v,u) exist in the graph, 
-    # attributes for the new undirected edge will be a combination of the attributes of the directed edges.
-    # if both edges exist in digraph and their edge data is different, 
-    # only one edge is created with an arbitrary choice of which edge data to use.
-    # you need to manually retain edges in both directions between nodes if their geometries are different
-    # this is necessary to save shapefiles for weird intersections like the one at 41.8958697,-87.6794924
-    # find all edges (u,v) that have a parallel edge going the opposite direction (v,u) with a different osmid
-    for u, v, key, data in G.edges(keys=True, data=True):
-        try:
-            # look at each edge going the opposite direction (from v to u)
-            for key2 in G.edge[v][u]:
-                # if this edge has geometry and its osmid is different from its reverse's
-                if 'geometry' in data and not data['osmid'] == G.edge[v][u][key2]['osmid']:
-                    # turn the geometry of each edge into lists of x's and y's
-                    geom1 = [list(coords) for coords in data['geometry'].xy]
-                    geom2 = [list(coords) for coords in G_undir[u][v][key]['geometry'].xy]
-                    # reverse the first edge's list of x's and y's to look for a match in either order
-                    geom1_r = [list(reversed(list(coords))) for coords in data['geometry'].xy]
-                    # if the edge's geometry doesn't match its reverse's geometry in either order
-                    if not (geom1 == geom2 or geom1_r == geom2):
-                        # add it as a new edge to the graph to be saved (with key equal to the current largest key plus one)
-                        new_key = max(G.edge[u][v]) + 1
-                        G_undir.add_edge(u, v, new_key, **data)
-        except:
-            pass
-    
-    return G_undir
+    # the previous operation added all directed edges from G as undirected edges in H
+    # this means we have duplicate edges for every bi-directional street
+    # so, look through the edges and remove any duplicates
+    duplicate_edges = []
+    for u, v, key, data in H.edges(keys=True, data=True):
 
+        # if we haven't already flagged this edge as a duplicate
+        if not (u, v, key) in duplicate_edges:
+
+            # look at this first edge's parallel edges one at a time
+            for key_other in H.edge[u][v]:
+
+                # don't compare this edge to itself
+                if not key_other == key:
+
+                    # compare the first edge's data to the second's to see if they are duplicates
+                    if is_duplicate_edge(data, H.edge[u][v][key_other]):
+
+                        # if they match up, flag the duplicate for removal
+                        duplicate_edges.append((u, v, key_other))
+                        
+    H.remove_edges_from(duplicate_edges)
+    log('Made undirected graph in {:,.2f} seconds'.format(time.time() - start_time))
     
+    return H
+
+
 def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geometry=True):
     """
     Convert a graph into node and/or edge GeoDataFrames
-    
+
     Parameters
     ----------
     G : networkx multidigraph
@@ -313,7 +380,7 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
         if True, create a geometry column from node x and y data
     fill_edge_geometry : bool
         if True, fill in missing edge geometry fields using origin and destination nodes
-    
+
     Returns
     -------
     GeoDataFrame or tuple
@@ -322,13 +389,13 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
 
     if not (nodes or edges):
         raise ValueError('You must request nodes or edges, or both.')
-    
+
     to_return = []
 
     if nodes:
-        
+
         start_time = time.time()
-        
+
         nodes = {node:data for node, data in G.nodes(data=True)}
         gdf_nodes = gpd.GeoDataFrame(nodes).T
         if node_geometry:
@@ -336,14 +403,14 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
         gdf_nodes.crs = G.graph['crs']
         gdf_nodes.gdf_name = '{}_nodes'.format(G.graph['name'])
         gdf_nodes['osmid'] = gdf_nodes['osmid'].astype(np.int64).map(make_str)
-        
+
         to_return.append(gdf_nodes)
         log('Created GeoDataFrame "{}" from graph in {:,.2f} seconds'.format(gdf_nodes.gdf_name, time.time()-start_time))
 
     if edges:
-        
+
         start_time = time.time()
-        
+
         # create a list to hold our edges, then loop through each edge in the graph
         edges = []
         for u, v, key, data in G.edges(keys=True, data=True):
@@ -361,41 +428,41 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
                     edge_details['geometry'] = LineString([point_u, point_v])
                 else:
                     edge_details['geometry'] = np.nan
-            
+
             edges.append(edge_details)
-        
+
         # create a GeoDataFrame from the list of edges and set the CRS
         gdf_edges = gpd.GeoDataFrame(edges)
         gdf_edges.crs = G.graph['crs']
         gdf_edges.gdf_name = '{}_edges'.format(G.graph['name'])
-        
+
         to_return.append(gdf_edges)
         log('Created GeoDataFrame "{}" from graph in {:,.2f} seconds'.format(gdf_edges.gdf_name, time.time()-start_time))
-        
+
     if len(to_return) > 1:
         return tuple(to_return)
     else:
         return to_return[0]
-    
+
 
 def gdfs_to_graph(gdf_nodes, gdf_edges):
     """
     Convert node and edge GeoDataFrames into a graph
-    
+
     Parameters
     ----------
     gdf_nodes : GeoDataFrame
     gdf_edges : GeoDataFrame
-    
+
     Returns
     -------
     networkx multidigraph
     """
-    
+
     G = nx.MultiDiGraph()
     G.graph['crs'] = gdf_nodes.crs
     G.graph['name'] = gdf_nodes.gdf_name.rstrip('_nodes')
-    
+
     # add the nodes and their attributes to the graph
     G.add_nodes_from(gdf_nodes.index)
     attributes = gdf_nodes.to_dict()
@@ -403,7 +470,7 @@ def gdfs_to_graph(gdf_nodes, gdf_edges):
         # only add this attribute to nodes which have a non-null value for it
         attribute_values = {k:v for k, v in attributes[attribute_name].items() if pd.notnull(v)}
         nx.set_node_attributes(G, attribute_name, attribute_values)
-    
+
     # add the edges and attributes that are not u, v, key (as they're added separately) or null
     for _, row in gdf_edges.iterrows():
         attrs = {}
@@ -411,19 +478,19 @@ def gdfs_to_graph(gdf_nodes, gdf_edges):
             if (label not in ['u', 'v', 'key']) and (isinstance(value, list) or pd.notnull(value)):
                 attrs[label] = value
         G.add_edge(u=row['u'], v=row['v'], key=row['key'], **attrs)
-    
-    return G    
 
-    
+    return G
+
+
 def make_shp_filename(place_name):
     """
     Create a filename string in a consistent format from a place name string.
-    
+
     Parameters
     ----------
     place_name : string
         place name to convert into a filename
-    
+
     Returns
     -------
     string
@@ -432,4 +499,3 @@ def make_shp_filename(place_name):
     filename = '-'.join(name_pieces).lower().replace(' ','_')
     filename = re.sub('[^0-9a-zA-Z_-]+', '', filename)
     return filename
-
